local _, ns = ...
local M, R, U, I = unpack(ns)
local module = M:RegisterModule("Actionbar")
local cfg = R.bars.bar1

function module:OnLogin()
	if not MaoRUISettingDB["Actionbar"]["Enable"] then return end

	local padding, margin = 2, 2
	local num = NUM_ACTIONBAR_BUTTONS
	local buttonList = {}
	local layout = MaoRUISettingDB["Actionbar"]["Styles"]

	--create the frame to hold the buttons
	local frame = CreateFrame("Frame", "NDui_ActionBar1", UIParent, "SecureHandlerStateTemplate")
	frame:SetWidth(num*cfg.size + (num-1)*margin + 2*padding)
	frame:SetHeight(cfg.size + 2*padding)
	if layout == 5 then
		frame.Pos = {"BOTTOM", UIParent, "BOTTOM", -108, 2}
	elseif (layout == 6) or (layout == 10) then
		frame.Pos = {"CENTER", UIParent, "CENTER", 80, -220}
	elseif layout == 9 then
	  frame:SetWidth(num*cfg.size + (num-1)*margin + 2*padding)
	  frame:SetHeight(cfg.size + 2*padding)
		frame.Pos = {"CENTER", UIParent, "CENTER", 76, -220}
	else
		frame.Pos = {"BOTTOM", UIParent, "BOTTOM", 0, 2}
	end
	frame:SetScale(cfg.scale)

	for i = 1, num do
		local button = _G["ActionButton"..i]
		table.insert(buttonList, button) --add the button object to the list
		button:SetParent(frame)
		button:SetSize(cfg.size, cfg.size)
		button:ClearAllPoints()
		if layout == 6 then
		  if i == 1 then
			  button:SetPoint("LEFT", frame, padding, 0)
		  elseif i == 9 then
			  local previous = _G["MultiBarRightButton1"]
			  button:SetPoint("RIGHT", previous, "BOTTOMLEFT", -margin, 0)
		  elseif i == 10 then
			  local previous = _G["ActionButton9"]
			  button:SetPoint("RIGHT", previous, "BOTTOMLEFT", -margin, 0)
			elseif i == 11 then
			  local previous = _G["MultiBarRightButton9"]
			  button:SetPoint("LEFT", previous, "BOTTOMRIGHT", margin, 0)
			elseif i == 12 then
			  local previous = _G["ActionButton11"]
			  button:SetPoint("LEFT", previous, "BOTTOMRIGHT", margin, 0)
			else
			  local previous = _G["ActionButton"..i-1]
			  button:SetPoint("LEFT", previous, "RIGHT", margin, 0)
			end
		elseif layout == 9 then
		  if i == 1 then
			  button:SetPoint("LEFT", frame, padding, 0)
		  elseif i == 9 then
			  local previous = _G["ActionButton1"]
			  button:SetPoint("TOPRIGHT", previous, "BOTTOM", -margin+10, 0)
			elseif i == 11 then
			  local previous = _G["ActionButton7"]
			  button:SetPoint("TOPLEFT", previous, "BOTTOM", margin-10, 0)
			else
			  local previous = _G["ActionButton"..i-1]
			  button:SetPoint("LEFT", previous, "RIGHT", margin, 0)
			end
		elseif layout == 10 then
		  if i == 1 then
			  button:SetPoint("LEFT", frame, padding, 0)
		  elseif i == 9 then
			  local previous = _G["MultiBarRightButton1"]
			  button:SetPoint("RIGHT", previous, "LEFT", -margin, 0)
		  elseif i == 10 then
			  local previous = _G["ActionButton9"]
			  button:SetPoint("TOP", previous, "BOTTOM", 0, -margin)
			elseif i == 11 then
			  local previous = _G["MultiBarRightButton9"]
			  button:SetPoint("LEFT", previous, "RIGHT", margin, 0)
			elseif i == 12 then
			  local previous = _G["ActionButton11"]
			  button:SetPoint("TOP", previous, "BOTTOM", 0, -margin)
			else
			  local previous = _G["ActionButton"..i-1]
			  button:SetPoint("LEFT", previous, "RIGHT", margin, 0)
			end
		else
		  if i == 1 then
			  button:SetPoint("BOTTOMLEFT", frame, padding, padding)
		  else
			  local previous = _G["ActionButton"..i-1]
			  button:SetPoint("LEFT", previous, "RIGHT", margin, 0)
		  end
		end
	end

	--show/hide the frame on a given state driver
	frame.frameVisibility = "[petbattle] hide; show"
	RegisterStateDriver(frame, "visibility", frame.frameVisibility)

	--create drag frame and drag functionality
	if R.bars.userplaced then
		M.Mover(frame, "Main Actionbar", "Bar1", frame.Pos)
	end

	--create the mouseover functionality
	if cfg.fader then
		M:CreateButtonFrameFader(frame, buttonList, cfg.fader)
	end

	--fix stupid blizzard
	local function ToggleButtonGrid()
		if InCombatLockdown() then return end
		local showgrid = tonumber(GetCVar("alwaysShowActionBars"))
		for _, button in next, buttonList do
			button:SetAttribute("showgrid", showgrid)
			ActionButton_ShowGrid(button)
		end
	end
	hooksecurefunc("MultiActionBar_UpdateGridVisibility", ToggleButtonGrid)

	--_onstate-page state driver
	local actionPage = "[bar:6]6;[bar:5]5;[bar:4]4;[bar:3]3;[bar:2]2;[overridebar]14;[shapeshift]13;[vehicleui]12;[possessbar]12;[bonusbar:5]11;[bonusbar:4]10;[bonusbar:3]9;[bonusbar:2]8;[bonusbar:1]7;1"
	local buttonName = "ActionButton"
	for i, button in next, buttonList do
		frame:SetFrameRef(buttonName..i, button)
	end

	frame:Execute(([[
		buttons = table.new()
		for i = 1, %d do
			table.insert(buttons, self:GetFrameRef("%s"..i))
		end
	]]):format(num, buttonName))

	frame:SetAttribute("_onstate-page", [[
		for i, button in next, buttons do
			button:SetAttribute("actionpage", newstate)
		end
	]])
	RegisterStateDriver(frame, "page", actionPage)

	--add elements
	self:CreateBar2()
	self:CreateBar3()
	self:CreateBar4()
	self:CreateBar5()
	self:CreateExtrabar()
	self:CreateLeaveVehicle()
	self:CreatePetbar()
	self:CreateStancebar()
	self:HideBlizz()
	self:ReskinBars()
end