﻿local _, ns = ...
local M, R, U, I = unpack(ns)
local module = M:RegisterModule("Maps")

local mapRects = {}
local tempVec2D = CreateVector2D(0, 0)

function module:GetPlayerMapPos(mapID)
	tempVec2D.x, tempVec2D.y = UnitPosition("player")
	if not tempVec2D.x then return end

	local mapRect = mapRects[mapID]
	if not mapRect then
		mapRect = {}
		mapRect[1] = select(2, C_Map.GetWorldPosFromMapPos(mapID, CreateVector2D(0, 0)))
		mapRect[2] = select(2, C_Map.GetWorldPosFromMapPos(mapID, CreateVector2D(1, 1)))
		mapRect[2]:Subtract(mapRect[1])

		mapRects[mapID] = mapRect
	end
	tempVec2D:Subtract(mapRect[1])

	return tempVec2D.y/mapRect[2].y, tempVec2D.x/mapRect[2].x
end

function module:OnLogin()
	local WorldMapFrame = _G.WorldMapFrame
	local BorderFrame = WorldMapFrame.BorderFrame
	local mapBody = WorldMapFrame:GetCanvasContainer()
	local formatText = I.MyColor..": %.1f, %.1f"
	local scale = mapBody:GetEffectiveScale()
	local width, height, mapID = mapBody:GetWidth(), mapBody:GetHeight()
	BorderFrame.Tutorial:SetPoint("TOPLEFT", WorldMapFrame, "TOPLEFT", -12, -12)

	-- Scaling
	if not WorldMapFrame.isMaximized then WorldMapFrame:SetScale(MaoRUISettingDB["Map"]["MapScale"]) end
	hooksecurefunc(WorldMapFrame, "Minimize", function(self)
		if InCombatLockdown() then return end
		self:SetScale(MaoRUISettingDB["Map"]["MapScale"])
	end)
	hooksecurefunc(WorldMapFrame, "Maximize", function(self)
		if InCombatLockdown() then return end
		self:SetScale(1)
	end)

	-- Generate Coords
	if not MaoRUISettingDB["Map"]["Coord"] then return end
	local player = M.CreateFS(BorderFrame, 12, "", false, "BOTTOMLEFT", 170, 0)
	local cursor = M.CreateFS(BorderFrame, 12, "", false, "BOTTOMLEFT", 285, 0)

	hooksecurefunc(WorldMapFrame, "OnFrameSizeChanged", function()
		width, height = mapBody:GetWidth(), mapBody:GetHeight()
	end)
	hooksecurefunc(WorldMapFrame, "OnMapChanged", function(self)
		if self:GetMapID() == C_Map.GetBestMapForUnit("player") then
			mapID = self:GetMapID()
		else
			mapID = nil
		end
	end)

	local function CursorCoords()
		local left, top = mapBody:GetLeft() or 0, mapBody:GetTop() or 0
		local x, y = GetCursorPosition()
		local cx = (x/scale - left) / width
		local cy = (top - y/scale) / height
		if cx < 0 or cx > 1 or cy < 0 or cy > 1 then return end
		return cx, cy
	end

	local function CoordsUpdate(player, cursor)
		local cx, cy = CursorCoords()
		if cx and cy then
			cursor:SetFormattedText(MOUSE_LABEL..formatText, 100 * cx, 100 * cy)
		else
			cursor:SetText(MOUSE_LABEL..I.MyColor..": |cffff0000^-^|r")
		end
		if not mapID then
			player:SetText(PLAYER..I.MyColor..": |cffff0000^-^|r")
		else
			local x, y = self:GetPlayerMapPos(mapID)
			if not x or (x == 0 and y == 0) then
				player:SetText(PLAYER..I.MyColor..": |cffff0000^-^|r")
			else
				player:SetFormattedText(PLAYER..formatText, 100 * x, 100 * y)
			end
		end
	end

	local function UpdateCoords(self, elapsed)
		self.elapsed = (self.elapsed or 0) + elapsed
		if self.elapsed > .1 then
			CoordsUpdate(player, cursor)
			self.elapsed = 0
		end
	end

	local updater = CreateFrame("Frame")
	updater:SetScript("OnUpdate", UpdateCoords)
	updater:Hide()

	local function isUpdating(self)
		updater:SetShown(self:IsShown())
	end
	WorldMapFrame:HookScript("OnShow", isUpdating)
	WorldMapFrame:HookScript("OnHide", isUpdating)
end